// ============================================================================
// Project:	A Tiny DBMS (Exercise for Course "DBMS-Implementation")
// Filename:	check.h
// Purpose:	Runtime Assertion Check
// Last Change:	01.11.2011
// Language:	C++
// EMail:	brass@informatik.uni-halle.de
// WWW:		http://www.informatik.uni-halle.de/~brass/
// Address:	Feldschloesschen 15, D-06120 Halle (Saale), GERMANY
// Copyright:	(c) 1997-2011 by Stefan Brass
// License:	See file "LICENSE" for copying conditions.
// Note:	There is no warranty at all - this code may contain bugs.
// ============================================================================


//=============================================================================
// Include File Frame:
//=============================================================================

#ifndef	CHECK_INCLUDED
#define CHECK_INCLUDED

//=============================================================================
// Used Types and Macros:
//=============================================================================

#ifndef VER_INCLUDED
#include "ver.h"
#endif

#ifndef STR_INCLUDED
#include "str.h"
#endif


//=============================================================================
// Exported Functions:
//=============================================================================

//-----------------------------------------------------------------------------
// check_init: Catch "segmentation fault", print error message to stdout.
//-----------------------------------------------------------------------------

void	check_init(void);

//-----------------------------------------------------------------------------
// CHECK: Check whether an expression is true.
//-----------------------------------------------------------------------------

#ifdef	VER_DEBUG
#define	CHECK(EX, MSG) \
	(void)((EX) || (check_failed(__FILE__, __LINE__, MSG), 0))

#else
#define	CHECK(EX, MSG) \
	((void)0)
#endif

//-----------------------------------------------------------------------------
// CHECK_VALID: Call integrity check for current class.
//-----------------------------------------------------------------------------

#ifdef	VER_DEBUG
#define	CHECK_VALID(PLACE) \
	check_emsg(__FILE__, __LINE__, PLACE, valid())
#else
#define	CHECK_VALID(EX) \
	((void)0)
#endif

//-----------------------------------------------------------------------------
// CHECK_PAR: Check parameter.
//-----------------------------------------------------------------------------

#ifdef	VER_DEBUG
#define	CHECK_PAR(PAR, PLACE) \
	check_emsg(__FILE__, __LINE__, PLACE, (PAR)->valid())
#else
#define	CHECK_PAR(PAR, PLACE) \
	((void)0)
#endif

//-----------------------------------------------------------------------------
// CHECK_IMPOSSIBLE: Report an error.
//-----------------------------------------------------------------------------

#ifdef	VER_DEBUG
#define	CHECK_IMPOSSIBLE(MSG) \
	check_failed(__FILE__, __LINE__, MSG)
#else
#define	CHECK_IMPOSSIBLE(MSG) \
	((void)0)
#endif

//-----------------------------------------------------------------------------
// CHECK_CODE: Code which is only used for assertion checking.
//-----------------------------------------------------------------------------

#ifdef	VER_DEBUG
#define	CHECK_CODE(CODE) \
	CODE
#else
#define	CHECK_CODE(CODE)
#endif

//-----------------------------------------------------------------------------
// check_failed: Print error message and abort program.
//-----------------------------------------------------------------------------

#ifdef	VER_DEBUG
void	check_failed(str_t file, int line, str_t msg);
#endif

//-----------------------------------------------------------------------------
// check_emsg: Check whether error message is not nil, print and abort if yes.
//-----------------------------------------------------------------------------

#ifdef	VER_DEBUG
void	check_emsg(str_t file, int line, str_t place, str_t msg);
#endif

//-----------------------------------------------------------------------------
// check_on_exit: Define a function to be called when check_failed aborts.
//-----------------------------------------------------------------------------

#ifdef	VER_DEBUG
void	check_on_exit(void (*f)());
#endif

//=============================================================================
// End of File:
//=============================================================================

#endif

